/*================================================================ 
 	Copyright 2008, Yoav Goldberg.
 	
 	This file is part of splitSVM.
 
    splitSVM is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    splitSVM is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with splitSVM.  If not, see http://www.gnu.org/licenses/.
     
 ================================================================*/
package yg.classifier.fmap;

import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;


//import classifier.fastsvm.io.FeatureMapReader;

public class FastFeatureMap {

	private Map<String, ArrayList<ArrayList<Integer>>> featMap;
	
	public FastFeatureMap(FeatureMap fm) throws IOException {
		featMap = new HashMap<String, ArrayList<ArrayList<Integer>>>();
		
		this.init(fm);
									
	}
	
	private void init(FeatureMap fm) {
		int MIN_LOCATION = 0;
		int MAX_LOCATION = 0;
		int MAX_TYPE = 0;
		for (String feat : fm.features()) {
			String[] parts = feat.split(":");
			int location = Integer.parseInt(parts[1].replace("+", "0"));
			int type = Integer.parseInt(parts[2]);
			if (type > MAX_TYPE) 			{ MAX_TYPE = type; }
			if (location > MAX_LOCATION) 	{ MAX_LOCATION = location; }
			if (location < MIN_LOCATION) 	{ MIN_LOCATION = location; }
		}
		int WINDOW_SIZE = (-MIN_LOCATION) + MAX_LOCATION + 1;
		System.out.println("window size:"+WINDOW_SIZE);
		System.out.println("max type:"+MAX_TYPE);
		System.out.println("min location:"+MIN_LOCATION);
		
		for (String feat : fm.features()) {
			//System.out.println("feat: [" + feat +"]");
			final int featNum = fm.getNum(feat);			
			final String[] parts = feat.split(":", 4);
			int location = Integer.parseInt(parts[1].replace("+", "0"))+(-MIN_LOCATION);
			int type = Integer.parseInt(parts[2]);
			String featStr = parts[3].intern();
			//System.out.println("loc:" + location + " type:" + type);
			// add the type,location,featStr -> featNum mapping
			ArrayList<ArrayList<Integer>> typeLocationFNum = this.featMap.get(featStr);
			if (typeLocationFNum == null) {
				typeLocationFNum = new ArrayList<ArrayList<Integer>>(MAX_TYPE);
				for (int i=0; i<=MAX_TYPE; i++) {typeLocationFNum.add(i, null);}
				featMap.put(featStr, typeLocationFNum);			
			}
			ArrayList<Integer> locationFNum = null;			
			locationFNum = typeLocationFNum.get(type);
			if (locationFNum == null) { 			
				locationFNum = new ArrayList<Integer>(WINDOW_SIZE);
				for (int i=0; i<=WINDOW_SIZE; i++) {	locationFNum.add(i, null); }
				typeLocationFNum.set(type, locationFNum);
			}			
			locationFNum.set(location, featNum);									
		}
	}

	public PartialFeature getPartialFeature(int type, String feat) {
		ArrayList<ArrayList<Integer>> typeLocationFNum = featMap.get(feat);
		if (typeLocationFNum == null) {
			return null;
		}
		return new PartialFeature(typeLocationFNum.get(type), type, feat);		
	}

	////////////////////////////////////////////////////////////////////////
	public class PartialFeature {
		private ArrayList<Integer> locationFNum = null;
		
		private String featStr = null;
		private int type;
		
		public PartialFeature(ArrayList<Integer> locationFNum, int type, String featStr) {
			this.locationFNum = locationFNum;
			this.featStr = featStr;
			this.type = type;
		}
		
		public int getFNumForLocation(int location) {
			if (this.locationFNum == null) {
				//System.out.println("-1 feature:" + this.featStr + ":" + type);
				return -1;
			}
			Integer ret = this.locationFNum.get(location);
			if (ret == null) {
				//vohmm.util.Logger.getLogger().println("-1 feature_2:" + this.featStr + ":" + type + ":" + location);
				return -1; 
			}
			return ret;
		}
		
		public String toString() {
			return "PartialFeature:LOC:"+this.type+":"+this.featStr;
		}
	}

	
}


