/*================================================================ 
 	Copyright 2008, Yoav Goldberg.
 	
 	This file is part of splitSVM.
 
    splitSVM is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    splitSVM is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with splitSVM.  If not, see http://www.gnu.org/licenses/.
     
 ================================================================*/
package yg.classifier.fmap;

import java.io.BufferedReader;
import java.io.IOException;
import java.util.Map;
import java.util.Set;

import yg.classifier.splitsvm.io.FeatureMapReader;



public class FeatureMap {

	private Map<String, Integer> featToInt;
	private Map<Integer, String> intToFeat;
	
	public FeatureMap(BufferedReader br, FeatureMapReader fr) throws IOException {
		fr.read(br);
		this.featToInt = fr.getFeatToNum();
		this.intToFeat = fr.getNumToFeat();							
	}
	
	public Set<String> features() {
		return featToInt.keySet();
	}
	

	public int getNum(String feat) {
		Integer res = this.featToInt.get(feat);
		if (res == null) {
			return -1;
		}		
		return res;
	}
	
	public int getNum(String symbol, int type, int index) {		
		if (index >= 0) {
			return this.getNum("F:+" + index + ":" + type + ":" + symbol);
		} else { 
			return this.getNum("F:" + index + ":" + type + ":" + symbol);
		}
	}
	
	public String numToString(int featureNum) {
		return this.intToFeat.get(featureNum);
	}
	
}
