/*================================================================ 
 	Copyright 2008, Yoav Goldberg.
 	
 	This file is part of splitSVM.
 
    splitSVM is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    splitSVM is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with splitSVM.  If not, see http://www.gnu.org/licenses/.
     
 ================================================================*/
package yg.classifier.splitsvm;

public abstract class MulticlassResolver {
	
	LabelsData labelsData = null;
	
	int  nrClassifiers;
	int  nrClasses;
	
	int[] vote;
	int[] voteIfFalse; // cid --> which ID to vote if false
	int[] voteIfTrue;  // cid --> which ID to vote if true

	
	public MulticlassResolver(LabelsData labels) {
		this.labelsData = labels;
		
		nrClassifiers = this.labelsData.falseLabels.length;
		nrClasses     = this.labelsData.labels.length;
		
		vote = new int[nrClasses];		
		voteIfFalse = new int[nrClassifiers]; // 
		voteIfTrue  = new int[nrClassifiers];
	}
	
	abstract int resolveToClassifierId(double scores[]);

	public String resolveToLabel(double scores[]) {
		return this.labelsData.labels[this.resolveToClassifierId(scores)];		
	}
	
	public int findIdxWithMaxValue(int[] vote) {
		int max = 0;
		int ret = -1;
		for (int i=0;i<vote.length;i++) {
			if (vote[i] >= max) {
				max = vote[i];
				ret = i;
			}
		}
		return ret;
	}
	
	LabelsData getLabelsData() {
		return this.labelsData;
	}
}
