/*================================================================ 
 	Copyright 2008, Yoav Goldberg.
 	
 	This file is part of splitSVM.
 
    splitSVM is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    splitSVM is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with splitSVM.  If not, see http://www.gnu.org/licenses/.
     
 ================================================================*/
package yg.classifier.splitsvm;

import java.util.Arrays;

public class OVRMulticlassResolver extends MulticlassResolver {

		
	/**
	 * 
	 * @param labels labels data
	 * @param otherLabel the label for the "other" class in the one-vs-others classification
	 */
	public OVRMulticlassResolver(LabelsData labels, String otherLabel) {
		super(labels);
		//System.out.println("OVR");
		this.initialize(otherLabel);
	}
	
	
	private void initialize(String otherLabel) {
				
		for (int i = 0; i < nrClassifiers; i++) {
			String falseLabel = this.labelsData.getFalse(i);
			String trueLabel  = this.labelsData.getTrue(i);
						
			voteIfFalse[i] = falseLabel.equals(otherLabel) ? -1 : this.labelsData.idxForLabel(falseLabel);			
			voteIfTrue[i]  = trueLabel.equals(otherLabel)  ? -1 : this.labelsData.idxForLabel(trueLabel);
		}		  		
	}
	
	public int resolveToClassifierId(double scores[]) {
		// initialize vote to 0
		Arrays.fill(vote, 0);
		int v = -1;
		for (int i=0; i<nrClassifiers; i++) {
			if (scores[i] > 0) { 
				v = voteIfTrue[i]; 
			}
			else {
				v = voteIfFalse[i];
			}
			if (v != -1) {
				vote[v]++;
			}
		}	
		
		return this.findIdxWithMaxValue(vote);
	}


}
