/*================================================================ 
 	Copyright 2008, Yoav Goldberg.
 	
 	This file is part of splitSVM.
 
    splitSVM is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    splitSVM is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with splitSVM.  If not, see http://www.gnu.org/licenses/.
     
 ================================================================*/
package yg.classifier.splitsvm;

import java.util.ArrayList;

import gnu.trove.TIntDoubleHashMap;
import gnu.trove.TIntObjectHashMap;

public class PairWeights {
	
	final private TIntObjectHashMap<TIntDoubleHashMap> f_f_w;
		
	final private TIntObjectHashMap<TIntObjectHashMap<ArrayList<TypeClassifierWeight>>> tmp_f_f_cw;
	private boolean stale = true;
		
	private TypeClassifierWeight[] empty_cw = {};
	
	private int maxfid = 0;
	
	int[] fidmap = null;
	private TypeClassifierWeight[][][] pairsToWeightsMatrix = null;
	
	public PairWeights() {
		this.f_f_w = new TIntObjectHashMap<TIntDoubleHashMap>();
		this.tmp_f_f_cw = new TIntObjectHashMap<TIntObjectHashMap<ArrayList<TypeClassifierWeight>>>();	
	}
	
	public void add(int f1, int f2, double w) {
		if (! f_f_w.containsKey(f1)) {
			f_f_w.put(f1, new TIntDoubleHashMap());
		}
		f_f_w.get(f1).put(f2, w);		
	}
	
	public void add(int f1, int f2, int classifierNum, double w) {
		//if (w > -9999) return;
		if (w == 0) {return;}
		if (f1 > maxfid) { maxfid = f1;  }
		if (f2 > maxfid) { maxfid = f2;  }		
		
		stale = true;		
		if (! tmp_f_f_cw.containsKey(f1)) {			
			ArrayList<TypeClassifierWeight> lst = new ArrayList<TypeClassifierWeight>();			
			lst.add(new TypeClassifierWeight(classifierNum, w));
			TIntObjectHashMap<ArrayList<TypeClassifierWeight>> map = new TIntObjectHashMap<ArrayList<TypeClassifierWeight>>();
			map.put(f2,lst);			
			tmp_f_f_cw.put(f1, map);
		}
		else {
			if (! tmp_f_f_cw.get(f1).containsKey(f2)) {
				ArrayList<TypeClassifierWeight> lst = new ArrayList<TypeClassifierWeight>();
				lst.add(new TypeClassifierWeight(classifierNum, w));				
				tmp_f_f_cw.get(f1).put(f2,lst);
			} else {
				tmp_f_f_cw.get(f1).get(f2).add(new TypeClassifierWeight(classifierNum, w));
			}
			
		}
	}
	
	
	public void doneAddingWeights() {
		// create mapping from feature-id to small range of ids
		fidmap = new int[maxfid+1];
		int fidcnt = 0;
		for (int f1 : tmp_f_f_cw.keys()) {
			if (fidmap[f1]==0) { fidmap[f1] = ++fidcnt; }
			for (int f2 : tmp_f_f_cw.get(f1).keys()) {
				if (fidmap[f2]==0) { fidmap[f2] = ++fidcnt; }
			}
		}
		// create a matrix from pairs to data
		pairsToWeightsMatrix = new TypeClassifierWeight[fidcnt+1][fidcnt+1][];
		for (int f1 : tmp_f_f_cw.keys()) {		
			for (int f2 : tmp_f_f_cw.get(f1).keys()) {
				TypeClassifierWeight[] data = tmp_f_f_cw.get(f1).get(f2).toArray(empty_cw);
				pairsToWeightsMatrix[fidmap[f1]][fidmap[f2]] = data;				
			}
		}
		for (int i=0;i<fidcnt+1;i++) {
			for (int j=0;j<fidcnt+1;j++) {
				if (pairsToWeightsMatrix[i][j] == null) {
					pairsToWeightsMatrix[i][j] = empty_cw;
				}
			}
		}
		stale = false;
		// TODO: clear the tmp_f_f_wc to preserve space? 
	}
	
	public double get(int f1, int f2) {
		
		//if (f_f_w.containsKey(f1)) {
		try {
			return f_f_w.get(f1).get(f2);
		} catch (NullPointerException e) { return 0; }
		//}
		//else {
		//	System.out.println("Oh!");
		//	return 0;
		//}
	}

	public TypeClassifierWeight[] getWeights(int f1, int f2) {
		if (this.stale) { this.doneAddingWeights(); }
		
		if (f1 < fidmap.length && f2 < fidmap.length ) { 
			//if (pairsToWeightsMatrix[fidmap[f1]][fidmap[f2]] == empty_cw) {System.out.println("missing pair:" + f1 + "," + f2);}
			return pairsToWeightsMatrix[fidmap[f1]][fidmap[f2]];
		} else {
			//System.out.println("Pair out of bounds:" + f1 + "," + f2);
			return empty_cw;
		}
	}	
}
