/*================================================================ 
 	Copyright 2008, Yoav Goldberg.
 	
 	This file is part of splitSVM.
 
    splitSVM is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    splitSVM is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with splitSVM.  If not, see http://www.gnu.org/licenses/.
     
 ================================================================*/
package yg.classifier.splitsvm;

import java.util.Arrays;

public class PairwiseMulticlassResolver extends MulticlassResolver {

	public PairwiseMulticlassResolver(LabelsData labels) {
		super(labels);
		//System.out.println("PAIR");
		this.initialize();
	}

	
	
	private void initialize() {
		for (int i = 0; i < nrClassifiers; i++) {
			final String falseLabel = this.labelsData.getFalse(i);
			final String trueLabel  = this.labelsData.getTrue(i);
						
			voteIfFalse[i] = this.labelsData.idxForLabel(falseLabel);			
			voteIfTrue[i]  = this.labelsData.idxForLabel(trueLabel);
		}			
	}



	@Override
	int resolveToClassifierId(double[] scores) {
		Arrays.fill(vote, 0);
		int v = -1;
		for (int i=0; i<nrClassifiers; i++) {
			if (scores[i] > 0) { 
				v = voteIfTrue[i]; 
			} else {
				v = voteIfFalse[i];
			}
			vote[v]++;
		}	
		
		return this.findIdxWithMaxValue(vote);
	}


}
