/*================================================================ 
 	Copyright 2008, Yoav Goldberg.
 	
 	This file is part of splitSVM.
 
    splitSVM is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    splitSVM is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with splitSVM.  If not, see http://www.gnu.org/licenses/.
     
 ================================================================*/
package yg.classifier.splitsvm;

import gnu.trove.TIntHashSet;

import java.io.BufferedReader;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.PrintWriter;
import java.util.Arrays;

import libsvm.svm_node;

public class ProbbedMulticlassModel extends FastMulticlassModel {
	
	private PrintWriter out = null;
	private Filter filter;
	
	protected boolean atDecisionLevel;
	
	public ProbbedMulticlassModel(String outfile, String filterFeaturesFile) {
		this(outfile, filterFeaturesFile, false);
	}
	
	public ProbbedMulticlassModel(String outfile, String filterFeaturesFile, boolean atDecisionLevel) {		
		super();
		System.out.println("--------------------------------");
		System.out.println("PROBED SVM MODEL: THIS IS SLOW! ");
		System.out.println("--------------------------------");
		try {
			if (outfile.equals("-")) {
				out = new PrintWriter(System.out);
			} else {
				if (atDecisionLevel) { outfile = outfile + ".dec"; }
				out = new PrintWriter(new FileOutputStream(outfile),true);
				System.out.println("out initialized:" + outfile);
			}			
			filter = new FileFeatureListFilter(filterFeaturesFile);
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		this.atDecisionLevel = atDecisionLevel;
	}
	
	public void getClassifiersScores(svm_node[] features, double[] scores) {		
		double[] tampered_scores = Arrays.copyOf(scores, scores.length);
		// original
		super.sepRareAndCommon(features);
		this.fillClassifierScores(scores);
		// with left-out features
		this.sepRareAndCommon(features);
		this.fillClassifierScores(tampered_scores);
		
		if (atDecisionLevel) {
			String tampdec = this.multiclassResolver.resolveToLabel(tampered_scores);
			String origdec = this.multiclassResolver.resolveToLabel(scores);
			out.print(origdec);
			out.print(" ");
			out.print(tampdec);
			out.print(" ");
			this.outputArray(out, features);
			out.println();
		} else {
			// compare and output
			int diffcount = 0;
			for (int i = 0; i< scores.length; i++) {
				if (scores[i]*tampered_scores[i] < 0) {
					diffcount++;
					out.print(i);
					out.print("-");
				}
			}
			
			//System.out.println("diff:" + diffcount + " " + this.modelFilePrefix);
			out.print("diff:" + diffcount);
			out.print(" features: ");
			this.outputArray(out, features);
			out.println();
		}
		
		
		
		
	}

	protected void sepRareAndCommon(svm_node[] features) {
		commonc = 0;
		rarec = 0;
		//System.out.println("features count:" + features.length);
		//System.out.print("Features:");
		for (int i=0;i<features.length;i++) {			
			//System.out.print(" " + features[i].index + ":" + features[i].value);
			int f = features[i].index;
			if (f < isCommonArray.length && isCommonArray[f]) {
				commons[commonc++] = f; 
			} else {
				if (this.shouldBeFiltered(f)) { continue; } // FILTER *********************
				rares[rarec++] = f;
				//System.out.println("i:" + i + " rare:"+f+" ");
			}
		}	
	}
	
	private void outputArray(PrintWriter out, svm_node[] arr) {
		for (int i=0;i<arr.length;i++) {
			if (filter.shouldBeFiltered(arr[i].index)) {
				out.print(arr[i].index);
				out.print(" ");
			}
		}
	}

	private boolean shouldBeFiltered(int i) {
		return filter.shouldBeFiltered(i);
	}


}

interface Filter {
	public boolean shouldBeFiltered(int feature);		
}

class FileFeatureListFilter implements Filter {

	TIntHashSet removeFeatures;
	
	public FileFeatureListFilter(String filename) throws IOException {
		removeFeatures = new TIntHashSet();
		BufferedReader b = new BufferedReader(new InputStreamReader(new FileInputStream(filename)) );
		String line = null;
		while (null != (line = b.readLine())  ) {
			removeFeatures.add(Integer.parseInt(line));
		}
	}
	public boolean shouldBeFiltered(int feature) {
		return removeFeatures.contains(feature);		
	}
	
}
