/*================================================================ 
 	Copyright 2008, Yoav Goldberg.
 	
 	This file is part of splitSVM.
 
    splitSVM is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    splitSVM is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with splitSVM.  If not, see http://www.gnu.org/licenses/.
     
 ================================================================*/
package yg.classifier.splitsvm;

import java.util.Arrays;

import gnu.trove.TIntArrayList;

public class SV {

	
	private int[] _features;
	public int[] _commonFeatures = null;
	public int[] _rareFeatures = null;
	private double _ai;
	
	private TypeClassifierAi[] _classifiersAis = null;
	

	public SV(double ai, int[] features) {
		this._ai = ai;
		this._features = features;
		//Arrays.sort(this._features);
		//this._featuresSet = new TIntHashSet(features);		
	}
	
	public SV(double[] ai, int[] classifiersNums, int[] feats) {
		_classifiersAis = new TypeClassifierAi[ai.length];
		for (int i=0;i<ai.length;i++) {
			_classifiersAis[i] = new TypeClassifierAi(classifiersNums[i],ai[i]);
		}
		_features = feats;
		Arrays.sort(this._features);		
	}

	public void initializeCommon(boolean[] commons) {
		TIntArrayList common = new TIntArrayList();
		TIntArrayList rare = new TIntArrayList();
		for (int feature : _features) {
			if (feature < commons.length && commons[feature]) {
				common.add(feature);
			} else {
				rare.add(feature);
			}
		}
		_commonFeatures = common.toNativeArray();
		_rareFeatures   = rare.toNativeArray();				
	}
	
	public String toString() {
		StringBuilder b = new StringBuilder();
		b.append("SV. Common:"+_commonFeatures.length + " Rare:" + _rareFeatures.length);
		b.append("\n\trare:");
		for (int i=0;i<_rareFeatures.length;i++) { b.append(" "+_rareFeatures[i]); } 
		b.append("\n\tcommon:");
		for (int i=0;i<_commonFeatures.length;i++) { b.append(" "+_commonFeatures[i]); } 				
		b.append("\n");
		return b.toString();
	}

	public double ai() { return _ai;}
	//public TIntHashSet featuresSet() { return _featuresSet; }
	public int[] features() { return _features; }
	public int[] commonFeatures() { return _commonFeatures; }
	public int[] rareFeatures() { return _rareFeatures; }

	public TypeClassifierAi[] classifiersAIs() {
		return _classifiersAis;
	}
}
