package yg.classifier.splitsvm.io;

import java.io.BufferedOutputStream;
import java.io.BufferedReader;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.DataOutputStream;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.util.StringTokenizer;
import java.util.zip.ZipEntry;
import java.util.zip.ZipInputStream;
import java.util.zip.ZipOutputStream;


public class MakeBinaryModel {

	protected static void convert_common_pairs_to_binary(InputStream is, OutputStream os) throws IOException {
		BufferedReader br = new BufferedReader(new InputStreamReader(is) );
		
		DataOutputStream out = new DataOutputStream(new BufferedOutputStream(os));
		
		String line = null;
			
		while (null != (line = br.readLine())) {
			StringTokenizer tok = new StringTokenizer(line," ");		
			
			// f1
			out.writeInt(Integer.parseInt(tok.nextToken()));
			// f2
			out.writeInt(Integer.parseInt(tok.nextToken()));
			
			// count of id / value
			out.writeShort(tok.countTokens());
			
			while (tok.hasMoreTokens()) {
				String[] parts = tok.nextToken().split(":");
				// id
				out.writeShort(Integer.parseInt(parts[0]));
				// val
				out.writeDouble(Double.parseDouble(parts[1]));	
			}
			out.flush();
		}		
	}
	
	protected static void convert_vecs_to_binary(InputStream is, OutputStream os) throws IOException {		
		BufferedReader br = new BufferedReader(new InputStreamReader(is) );
		DataOutputStream out = new DataOutputStream(new BufferedOutputStream(os));
		
		String line = null;
		int vs = 0;
		while (null != (line = br.readLine())) {
			vs++;
			String[] features = line.split(" ");
			out.writeShort(features.length);
			for (int i = 0; i < features.length; i++) {
				out.writeInt(Integer.parseInt(features[i]));
			}
		}
		out.flush();
		
		System.out.println("vecs count:" + vs);
	}
	
	protected static void convert_ais_to_binary(InputStream is, OutputStream os) throws IOException {
		BufferedReader br = new BufferedReader(new InputStreamReader(is) );
		DataOutputStream out = new DataOutputStream(new BufferedOutputStream(os));
			
		String line;
		int as = 0;
		while (null != (line = br.readLine())) {
			as++;
			StringTokenizer tok = new StringTokenizer(line," :");
			// count of id / value
			out.writeShort((tok.countTokens() / 2));
			
			while (tok.hasMoreTokens()) {
				// id
				out.writeShort(Integer.parseInt(tok.nextToken()));
				// val
				out.writeDouble(Double.parseDouble(tok.nextToken()));	
			}
		}
		out.flush();
		
		System.out.println("ais count:" + as);
	}
	
	
	protected static void streamCopy(InputStream is, OutputStream os) throws IOException {
		byte[] buf = new byte[2048];
		int len;
		while ((len = is.read(buf)) > 0) {
			os.write(buf, 0, len);
		}		
	}

	public static void binarizeZippedModel(String INFILE, String OUTFILE) throws FileNotFoundException, IOException {
		ByteArrayOutputStream out = new ByteArrayOutputStream(10000);
		ZipInputStream zin = new ZipInputStream(new FileInputStream(INFILE));
		ZipOutputStream zout = new ZipOutputStream(out);
		
		ZipEntry entry = zin.getNextEntry();
		while (entry != null) {
			String name = entry.getName();			
			if (name.equals("splt" + ModelReader.EXT_COMMONW)) {
				name = "splt" + FastMulticlassModelReader.EXT_COMMONW_BINARY;
				zout.putNextEntry(new ZipEntry(name));
				System.out.println("commonpairs -> binary");
				convert_common_pairs_to_binary(zin, zout);
				zout.closeEntry();
			} else if (name.equals("splt" + ModelReader.EXT_VECS)) {
				name = "splt" + FastMulticlassModelReader.EXT_VECS_BINARY;
				zout.putNextEntry(new ZipEntry(name));
				System.out.println("vecs -> binary");
				convert_vecs_to_binary(zin, zout);
				zout.closeEntry();
			} else if (name.equals("splt" + FastMulticlassModelReader.EXT_AIS)) {
				name = "splt" + FastMulticlassModelReader.EXT_AIS_BINARY;
				zout.putNextEntry(new ZipEntry(name));
				System.out.println("ais -> binary");
				convert_ais_to_binary(zin, zout);
				zout.closeEntry();
			} else {	
				// Transfer bytes from the ZIP file to the output file
				zout.putNextEntry(new ZipEntry(name));
				System.out.println("copy " + name);
				streamCopy(zin, zout);
				zout.closeEntry();
				System.out.println("done");
			}
			entry = zin.getNextEntry();
		}
		zin.close();
		zout.close();
		System.out.println("done");
		
		FileOutputStream fout = new FileOutputStream(OUTFILE);
		streamCopy(new ByteArrayInputStream(out.toByteArray()), fout);
		fout.close();
	}
	

	public static void main(String[] args) throws IOException {
		String INFILE = args[0];
		String OUTFILE = (args.length == 2) ? args[1] : args[0];
		
		binarizeZippedModel(INFILE, OUTFILE);
	}

}
