package yg.tagger;

import java.io.BufferedReader;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.Vector;


import yg.classifier.splitsvm.FastMulticlassModel;

public class YamchaFileTagger {
	
	SequentialTagger tagger;
	
	public YamchaFileTagger(FastMulticlassModel model) {
		this.tagger = new SequentialTagger(model);
	}

	private static Vector<String> readlines(BufferedReader br) throws IOException {
		Vector<String> ret = new Vector<String>();
		String line = null;
		
		while (null != (line = br.readLine())) {
			if (line.trim().equals("")) {
				break;
			}
			//System.out.println(line);
			ret.add(line);			
		}
		if (line == null) { return null; }
		return ret;
	}

	
	private ArrayList<String> tagLines(Vector<String> lines) {
		
		int numCols = lines.get(0).split(" ").length;
		
		String[][] splittedLines = new String[numCols][lines.size()];
		
		int lineId = 0;
		//System.out.println(""+lines.size());
		for (String line : lines) {
			String[] splitted = line.split(" ");
			for (int col=0;col<splitted.length; col++) {
				//System.out.println(lineId + ":" +line);
				splittedLines[col][lineId] = splitted[col];				
			}
			lineId++;
		}
		
		this.tagger.begin();
		for (int col=0;col<numCols;col++) {
			this.tagger.addFeaturesSequence(col, splittedLines[col]);
		}
		ArrayList<String> res = this.tagger.tag();
		
		return res;
	}

	public void tagFile(String filenameToTag, PrintWriter out) throws IOException {
		this.tagFile(new InputStreamReader(new FileInputStream(filenameToTag), "cp1255"),out);
		
	}
	public void tagFile(InputStreamReader in, PrintWriter out) throws IOException {
		
		BufferedReader br = new BufferedReader(in);
		//BufferedReader br2 = new BufferedReader(new InputStreamReader(new FileInputStream("ann.present_fixed-test.res"),"cp1255"));
		Vector<String> lines = new Vector<String>();
		ArrayList<String> results;
		//int good = 0, bad=0;		
		while (null != (lines = this.readlines(br))) {
			//Vector<String> lines2 = readlines(br2);
			results = this.tagLines(lines);			
			int i = 0;
			for (String line : lines) {
				//String[] parts = line.split(" ");
				//if (parts[parts.length-1].equals(results.get(i))) {
				//	good++;
				//} else { bad++;
					//System.out.println("MISTAKE:" + parts[parts.length-1] + " " + results.get(i));
					//String[] parts2 = lines2.get(i).split("\\s+");
					//if (!parts2[parts2.length-1].equals(results.get(i))) {
					//	System.out.println("MISTAKE:"+lines2.get(i));
					//	System.out.println("MISTAKE:" + parts2[parts2.length-1] + " " + results.get(i));
					//}
					
					
				//}
				out.print(line.trim());
				out.print("\t");
				out.println(results.get(i));				
				i++;
			}	
			out.println();
			
		}
		//System.out.println("g:" + good + " b:" + bad );
		out.flush();
		
	}

	public static void main(String[] args) throws IOException {
		
		String modelFilePrefix = args[0];
		String fileNameToTag   = args[1];
		
		FastMulticlassModel model = new FastMulticlassModel();
		model.read(modelFilePrefix);
		
		YamchaFileTagger tagger = new YamchaFileTagger(model);
		long start = System.currentTimeMillis();
		tagger.tagFile(fileNameToTag, new PrintWriter(System.out));
		System.out.println("tagging time:" + (System.currentTimeMillis() - start));
	}

}
