#================================================================ 
#   Copyright 2008, Yoav Goldberg.
#   
#   This file is part of splitSVM.
# 
#    splitSVM is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Lesser General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    splitSVM is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#   You should have received a copy of the GNU Lesser General Public License
#   along with splitSVM.  If not, see http://www.gnu.org/licenses/.
#    
#================================================================
import itertools
from compat import defaultdict

class FeatureVectorInterface:
   counter = itertools.count(0)
   def __init__(self, ai, feats, id=None):
      self._feats = feats
      self._ai = ai
      if not id: id = self.counter.next()
      self._id = id
      self._model_num_to_ai = defaultdict(float)

   def features(self):
      """
      return the list of features (each feature is an ID (int))
      """
      return self._feats

   def ai(self):
      """
      return the vector's weight (*label \in {-1,1} )
      """
      return self._ai

   def id(self):
      """
      returns a unique-id for the vector
      """
      return self._id

   def addAI(self, ai, classifierNum):
      self._model_num_to_ai[classifierNum] += ai

   def ais(self):
      return self._model_num_to_ai.items()
      
   
class ClassifierDataInterace:
   def __init__(self, vecs, thr, fids, true_label = '1', false_label = '-1'):
      self._vecs = vecs
      self._thr  = thr
      self._fids = fids
      self._true_label = true_label
      self._false_label = false_label
      self._degree = 2
      self._coef0 = 1.0
      self._gamma = 1.0

   def vecs(self): 
      """
      return a list of FeatureVectorInterface
      """
      return self._vecs

   def thrs(self):
      """
      the threshold for this classifier (b)
      """
      return self._thr

   def fids(self):
      """
      a list of the ids of features in the model
      """
      return self._fids

   def set_true_label(self, lbl): self._true_label = lbl
   def true_label(self): return self._true_label
   def set_false_label(self, lbl): self._false_label = lbl
   def false_label(self): return self._false_label

   def set_kernel_type(self, kernel_type): self._kernel_type = kernel_type
   def kernel_type(self): return self._kernel_type
   def set_degree(self, degree): self._degree = degree
   def degree(self): return self._degree
   def set_coef0(self, coef0): self._coef0 = coef0
   def coef0(self): return self._coef0
   def set_gamma(self, gamma): self._gamma = gamma
   def gamma(self): return self._gamma


### the next items are probably unused.

class ClassifiersCollectionException(Exception):pass

class ClassifiersCollectionInterface:
   def __init__(self):
      self.classifiers = {}

   def add_classifier(self,lbl,classifier):
      if lbl in self.classifiers:
         raise ClassifiersCollectionException("attempt to add a duplicate classifier: %s" % lbl)
      self.classifiers[lbl] = classifier

   def is_ovr(self): return False
   def is_pairwise(self): return False

   def features_map(self): return {}
   def features_ids(self): 
      fids = set()
      for cls in self.classifiers.values():
         fids.update(set(cls.fids()))
      return list(sorted(fids))
   def classifiers_map(self): return self.classifiers


