#================================================================ 
#   Copyright 2008, Yoav Goldberg.
#   
#   This file is part of splitSVM.
# 
#    splitSVM is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Lesser General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    splitSVM is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#   You should have received a copy of the GNU Lesser General Public License
#   along with splitSVM.  If not, see http://www.gnu.org/licenses/.
#    
#================================================================

## like io, with changes to the following functions:
## - write_threshold / read_threshold   (now has multiple thresholds)
## - write_common_pairs_weights / read  (now each line is "f1 f2 modelnum:w ...  "
## - write_vectors   / read_vectors     (now does not store ai
## - added write_ais / read_ais         (each line "modelid:ai ... "
import sys

def read_common_features(fname):
   return set([int(x) for x in file(fname).read().split()])

def write_common_features(fname, features_set):
   fout = file(fname,"w")
   fout.write(" ".join((str(x) for x in features_set)))
   fout.close()

def read_rare_features_to_vecs_map(fname):
   res = {}
   lines = file(fname)
   top_feature = lines.next()
   for line in lines:
      fnum,vecs = line.strip().split()
      vecs = vecs.strip().split(":")
      res[int(fnum)] = [int(x) for x in vecs]
   return res

def write_rare_features_to_vecs_map(fname, fs_to_vecnums_map):
   fout = file(fname,"w")
   try:
      max_feat = max(fs_to_vecnums_map.keys())
   except ValueError: max_feat = 0
   fout.write("%s\n" % max_feat)
   for fnum, vecs in fs_to_vecnums_map.iteritems():
      fout.write("%s %s\n" % (fnum, ":".join(map(str,sorted(vecs)))))
   fout.close()

def read_threshold(fname):
   return [double(i) for i in (file(fname).read().split())]

def write_threshold(fname, thrs):
   """
   thr: a map from classifier_id to threshold
   """
   thrs = thrs.items()
   for i in [cid for (cid,thr) in thrs]:
      int(i)
   thrs = sorted(thrs,key=lambda (cid,thr):cid)
   thrs = [str(thr) for (cid, thr) in thrs]
   fout = file(fname,"w")
   fout.write("%s\n" % " ".join(thrs));
   fout.close()

def read_common_pair_weights(fname):
   """
   ret: map[(f1,f2)] -> [(mid,w),...]
   """
   res = {}
   for line in file(fname):
      f1,f2,w = line.strip().split(None,3)
      ws = w.split()
      ws = [(int(model),float(w)) for (model,w) in (mw.split(":") for mw in ws)]
      res[(int(f1),int(f2))] = ws
   return res

def write_common_pairs_weights(fname, pair_weights_map):
   """
   pair_weights_map: map[(f1,f2)][classifierid]->w
   """
   fout = file(fname,"w")
   for (f1,f2), mw in pair_weights_map.iteritems():
      w = " ".join(("%s:%s" % (modelnum, w) for (modelnum,w) in mw.iteritems()))
      fout.write("%s %s %s\n" % (f1,f2,w))
   fout.close()

def read_common_features_weights(fname):
   res = {}
   for line in file(fname):
      f,w = line.strip().split()
      res[int(f)] = float(w)
   return res

def write_common_features_weights(fname, features_weights_map):
   fout = file(fname,"w")
   for f, w in features_weights_map.iteritems():
      fout.write("%s %s\n" % (f,w))
   fout.close()


def read_vectors(fname):
   vecs = list()
   ais  = list()
   for line in file(fname):
      feats = [int(x) for x in line.strip().split()]
      vecs.append(feats)
   return vecs

def write_vectors(fname, vecs):
   fout = file(fname,"w")
   for vec in vecs:
      fout.write(" ".join(map(str,vec.features())))
      fout.write("\n")
   fout.close()

def write_ais(fname, vecs):
   fout = file(fname,"w")
   for vec in vecs:
      line = " ".join(("%s:%s" % (mid,ai) for (mid, ai) in vec.ais()))
      fout.write("%s\n" % line)
   fout.close()
      
def read_ais(fname):
   ret = []
   for line in file(fname):
      line = line.strip().split()
      line = [mai.split(":") for mai in line]
      line = [(int(mid),float(ai)) for mai in line]
      ret.append(line)   
   return ret

def read_features_map(fname):
   res = {}
   for line in file(fname):
      fnum,f = line.strip().split()
      res[f] = int(fnum)
   return res

def write_features_map(fname, fmap):
   fout = file(fname,"w")
   for fnum,fkey in fmap.iteritems():
      fout.write("%s %s\n" % (fnum,fkey))
   fout.close()

def read_kernel_data(fname):
   return [double(x.strip()) for x in file(fname).readlines() if x]

def write_kernel_data(fname, deg,gam,coef0):
   fout = file(fname,"w")
   fout.write("\n".join([str(x) for x in (deg,gam,coef0)]))
   fout.close()




