#================================================================ 
#   Copyright 2008, Yoav Goldberg.
#   
#   This file is part of splitSVM.
# 
#    splitSVM is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Lesser General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    splitSVM is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#   You should have received a copy of the GNU Lesser General Public License
#   along with splitSVM.  If not, see http://www.gnu.org/licenses/.
#    
#================================================================
from compat import defaultdict
from interfaces import FeatureVectorInterface, ClassifierDataInterace #, ClassifiersCollectionInterface
import itertools

class LIBSVMFeatureVector(FeatureVectorInterface):
   def __init__(self, feats, id):
      FeatureVectorInterface.__init__(self, None, feats, id)

class LIBSVMClassifierData(ClassifierDataInterace):
   def __init__(self, SVs, map_cid_thr, fids, labels_to_cids, cids_to_labels):
      ClassifierDataInterace.__init__(self, SVs, None, fids, None, None)
      self._cid_to_thr = map_cid_thr
      self._labels_to_cids = labels_to_cids
      self._cids_to_labels = cids_to_labels
   def is_ovr(self): return False
   def is_pairwise(self): return True
   def features_map(self): return {}
         


#class LIBSVMClassifiersCollection(ClassifiersCollectionInterface): # Unsued (I hope)
#   def is_pairwise(self): return True

class LIBSVMReadException(Exception): pass

class LIBSVMReader:
   @classmethod
   def read(cls, filename):
      lines = file(filename)

      ## header
      svm_type = lines.next().split()[1]
      kernel_type = lines.next().split()[1]
      if kernel_type != "polynomial": 
         raise LIBSVMReadException("unsupported kernel")
      degree = int(lines.next().split()[1])
      gamma  = float(lines.next().split()[1])
      coef0  = float(lines.next().split()[1])
      nr_classes = int(lines.next().split()[1])
      total_svs = int(lines.next().split()[1])

      rho = lines.next().strip().split()
      if not rho[0] == 'rho': raise LIBSVMReadException("expected rho, got %s" % rho)
      rho = [float(b) for b in rho[1:]]  # thresholds for classifiers
      if nr_classes*(nr_classes-1)/2 != len(rho) : raise LIBSVMReadException("bad number of classes / rho")

      labels = lines.next().strip().split()
      if labels[0] != 'label': raise LIBSVMReadException("expected labels, found %s" % labels[0])
      labels = labels[1:]
      if nr_classes != len(labels):
         raise LIBSVMReadException("MISMATCH! nr classes: %s, len(labels) %s" % (nr_classes, len(labels)))

      nr_svs = [int(x) for x in lines.next().strip().split()[1:]]
      if total_svs != sum(nr_svs): raise LIBSVMReadException("MISMATCH: sum(nr_svs) != total_svs")

      SV = lines.next().strip()
      if SV != "SV": raise LIBSVMReadException("expecing SV, got %s" % SV)

      rho = iter(rho)
      classifier_id_to_thr = defaultdict(float)

      classifiers_ids = itertools.count(0)
      cids_to_labels = {}
      labels_to_cids = {}
      for label1 in labels:
         for label2 in labels[labels.index(label1)+1:]:
            #print (label1,label2)
            cid = classifiers_ids.next()
            cids_to_labels[cid] = (label1,label2)
            labels_to_cids[(label1,label2)] = cid
            classifier_id_to_thr[cid] = rho.next()
      if len(labels_to_cids.keys()) != nr_classes*(nr_classes-1)/2:
         raise "too many/few classifiers in labels_to_cids"
      if len(cids_to_labels.keys()) != nr_classes*(nr_classes-1)/2:
         raise "too many/few classifiers in cids_to_labels"
      veclist = []
      vec_idxes = itertools.count(0)
      for label,lblcnt in zip(labels,nr_svs):
         print "label",label,"nr_svs",lblcnt
         for x in xrange(lblcnt):
            line = lines.next().strip().split()
            ais = [float(ai) for ai in line[:nr_classes-1]]
            #if label != '1': print "num ais",len(ais)
            feats = [int(f.split(":")[0]) for f in line[nr_classes-1:]]
            vid = vec_idxes.next()
            vec = LIBSVMFeatureVector(feats, vid)
            veclist.append(vec)
            ais = iter(ais)
            #if label != '1': print "round"
            for label_i in labels:
               #print label, label_i
               if label_i == label:
                  #print "skipped"
                  continue
               #if label == '3': print label,label_i
               ai = ais.next()
               if float(ai) != 0.0:
                  #if not (label,label_i) in labels_to_cids: continue
                  if (label,label_i) in labels_to_cids:
                     vec.addAI(ai, labels_to_cids[(label,label_i)]) # TODO: change labels order, OR maybe order should be always fixed 
                  else:
                     vec.addAI(ai, labels_to_cids[(label_i,label)])
            try:
               ais.next()
               raise LIBSVMReadException("Missing ai!")
            except StopIteration: 
               #print "Great!"
               pass
      if len(veclist) != total_svs: raise LIBSVMReadException("Wrong number of SVs read")
      fids = set()
      for vec in veclist: fids.update(set(vec.features()))
      # by now we have:
      # @ classifier_id_to_thr: map: classifierID(int) --> threshold b(float) 
      # @ veclist: a list of all SVs.  Each SV has features()->list(int) and ais()->list((cid, ai))
      # @ labels_to_cids: map : (label1,label2) --> classifierID(int)
      # @ cids_to_labels: reverse of the above
      # @ fids : set: id(int) for all features active in any classifier
      classifier = LIBSVMClassifierData(veclist, classifier_id_to_thr, fids, labels_to_cids, cids_to_labels)
      classifier.set_kernel_type(kernel_type)
      classifier.set_degree(degree)
      classifier.set_gamma(gamma)
      classifier.set_coef0(coef0)

      classifier.nr_classes = nr_classes
      classifier.nr_classifiers = len(classifier_id_to_thr.keys())
      classifier.labels = labels

      return classifier


if __name__ == '__main__':
   #model = LIBSVMReader.read("../../exps/malt-english/conll-2007/test/libsvm.mod")
   model = LIBSVMReader.read("../../exps/malt-english/conll-2007/test_nogamma/libsvm.mod")
   #for l,c in model.classifiers_map().items(): print l,c


