#!/usr/bin/python

#================================================================ 
#   Copyright 2008, Yoav Goldberg.
#   
#   This file is part of splitSVM.
# 
#    splitSVM is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Lesser General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    splitSVM is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#   You should have received a copy of the GNU Lesser General Public License
#   along with splitSVM.  If not, see http://www.gnu.org/licenses/.
#    
#================================================================
import sys

class WrongUsage(Exception): pass

def usage():
   print """
   Usage: %s TYPE IN [OUT] [S] 

   \tTYPE: input model type. Either 'libsvm' or 'yamcha'
   \tIN  : input model file
   \tOUT : optional output file prefix.  Default: <IN>.splt
   \tS   : features appearing in less than <S>%% of the support-vectors
   \t      are considered rare.
   \t      Default: 0.5  (half a percent)
   \t      (smaller S means more common features -> bigger and faster models)
   """ % sys.argv[0]
   sys.exit(1)

def read_args():
   try:
      # type
      type = sys.argv[1]
      if type not in ['libsvm','yamcha']: raise WrongUsage()

      # IN
      infile = sys.argv[2]

      try:
         outfile = sys.argv[3]
      except IndexError:
         outfile = infile + ".splt"

      try:
         rare = int(100.0 / float(sys.argv[4]))
      except IndexError:
         rare = 200

      return type, infile, outfile, rare
   except IndexError: raise WrongUsage()
   except ValueError: raise WrongUsage()


if __name__ == '__main__':
   import splitsvmlib
   
   try:
      (type, model_infile, out_file_prefix, rare) = read_args()

      if type == 'yamcha':
         import yamchalib
         model,fmap = yamchalib.YAMCHAAsMulticlassReader.read(model_infile)

      if type == 'libsvm':
         import libsvmlib
         model = libsvmlib.LIBSVMReader.read(model_infile)

      splitsvmlib.convert_and_write_multiclass_models(model, out_file_prefix, rare,zip=True)

   except WrongUsage: usage()
