#================================================================ 
#   Copyright 2008, Yoav Goldberg.
#   
#   This file is part of splitSVM.
# 
#    splitSVM is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Lesser General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    splitSVM is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#   You should have received a copy of the GNU Lesser General Public License
#   along with splitSVM.  If not, see http://www.gnu.org/licenses/.
#    
#================================================================

## NOTE: differs from mk_fastsvmdata:
## - assumes the libsvm SV interface
## - pair_wieghts is not map[pair]->w  but map[pair][modelnum]->w
## - assumes multiple-thresholds in FastSVMDecoderData: thrs is map[classifier_id]->threshold 

# in the multiclass case, share the vectors.
# this means: 
# (1) one file for SVs. 
# (2) reverse index of features for each model. 
# (3) a file containing an sv_idx -> ai  for each model.
# the format of (3) is: each line is the corresponding sv_idx, and it has space separated ai() for each model

from io2 import *
from compat import defaultdict
import os
from zipfile import ZipFile, ZIP_DEFLATED



class FastSVMDecoderData: # {{{
   def __init__(self, vecs, thrs, feature_ids,S=500, degree=2,gamma=1.0,coef0=1.0):
      self.S = S
      self.initialize(vecs, thrs, feature_ids,degree,gamma,coef0)

   def _common_vs_rare_features(self, vecs, feature_ids): # {{{
      #print "vec:",vecs[0].features()
      #print "fids:",feature_ids
      feat_counts = defaultdict(int)
      for vnum, vec in enumerate(vecs):
         for fnum in vec.features():
            feat_counts[fnum]+=1
      feat_rare_thr = len(vecs) / self.S  
      if feat_rare_thr < 2: feat_rare_thr = 2
      #feat_rare_thr = 99999999
      print "NumSVs:",len(vecs)
      print "Feat threshold:",feat_rare_thr
      def is_rare_type(fnum): return feat_counts[fnum] < feat_rare_thr
      rare_features = set()
      common_features = set()
      for fnum in feature_ids:
         if is_rare_type(fnum):
            rare_features.add(fnum)
         else:
            common_features.add(fnum)
      print "common:",len(common_features),"rare:", len(rare_features)
      self._common = common_features
      self._rare   = rare_features
      #}}}

   def _make_rare_to_svs_index(self, vecs, rare_features): # {{{
      rare_feature_to_vec_map = defaultdict(set)
      for vnum, vec in enumerate(vecs):
         for fnum in vec.features():
            if fnum in rare_features: 
               rare_feature_to_vec_map[fnum].add(vnum)
      print "NumVecs:",len(vecs)
      counts = [len(vnums) for fnum,vnums in rare_feature_to_vec_map.iteritems()]
      print "AvgSVS:",sum(counts)/(len(rare_feature_to_vec_map.keys())+1)
      try:
         print "MinSVS:",min(counts)
      except ValueError: print "-"
      try:
         print "MaxSVS:",max(counts)
      except ValueError: print "-"
      self._rare_features_to_vec_map = rare_feature_to_vec_map
      # }}}

   def _precalc_common_features_ws(self, vecs, common_features): # {{{
      # precalc all pairs for common features
      pair_weights = defaultdict(lambda:defaultdict(float)) # [pair][modelnum] -> w
      #ones = defaultdict(float)
      gamma = self._gamma
      coef0 = self._coef0
      xixi_const = 2.0*coef0*gamma+(gamma*gamma)
      xixj_const = 2.0*gamma*gamma
      coef0sqr = coef0*coef0
      for vec in vecs:
         #for (modelnum,ai) in vec.ais(): ones[modelnum] += ai
         feats = [fnum for fnum in vec.features() if fnum in common_features]
         for i in xrange(len(feats)):
            for j in xrange(i,len(feats)):
               pair = (feats[i],feats[j])
               if i==j:
                  for (modelnum, ai) in vec.ais():
                     #if ai != 0: pair_weights[pair][modelnum] += 3*ai
                     if ai != 0: pair_weights[pair][modelnum] += xixi_const*ai
               else:
                  for (modelnum, ai) in vec.ais():
                     #if ai != 0: pair_weights[pair][modelnum] += 2*ai
                     if ai != 0: pair_weights[pair][modelnum] += xixj_const*ai
      print "empirical num of pairs:",len(pair_weights)
      #print ones.items()
      self._pair_weights = pair_weights
      # }}}

   def _update_threshold_with_constant_w(self, vecs):
      coef0sqr = self._coef0*self._coef0
      for vec in vecs:
         for (modelnum, ai) in vec.ais():
            self._threshold[modelnum] -= coef0sqr*ai

   def initialize(self, vecs, thrs, fids,deg,gam,coef0):
      self._degree = deg
      self._gamma = gam
      self._coef0 = coef0
      self._threshold = thrs
      self._vecs = vecs

      self._common_vs_rare_features(vecs, fids) # sets _common, _rare
      self._make_rare_to_svs_index(vecs, self._rare) # sets _rare_features_to_vec_map
      self._precalc_common_features_ws(vecs, self._common) # sets _pair_weights
      self._update_threshold_with_constant_w(vecs)

   def write_to_files(self, filename_prefix, zip=False):
      PREFIX = filename_prefix
      ### write files
      # b threhsold
      write_threshold("%s.thrs" % PREFIX, self._threshold)
      # vectors
      write_vectors("%s.vecs" % PREFIX, self._vecs)
      write_ais("%s.ais" % PREFIX, self._vecs)
      # common_features_numbers
      write_common_features("%s.common" % PREFIX, self._common)
      # rare feats to svs map
      write_rare_features_to_vecs_map("%s.rarerindex" % PREFIX, self._rare_features_to_vec_map)
      # common pairs weights
      write_common_pairs_weights("%s.commonpairs" % PREFIX, self._pair_weights)
      # common feature weights
      #write_common_features_weights("%s.commonfs" % PREFIX, self._fweights)
      # kernel
      write_kernel_data("%s.kernel" % PREFIX, self._degree, self._gamma, self._coef0)
      if zip:
         print "packing"
         zf = ZipFile("%s.spltmod" % PREFIX,"w", ZIP_DEFLATED)
         zf.write("%s.thrs" % PREFIX, "splt.thrs")
         zf.write("%s.vecs" % PREFIX, "splt.vecs")
         zf.write("%s.ais"  % PREFIX, "splt.ais")
         zf.write("%s.common" % PREFIX, "splt.common")
         zf.write("%s.rarerindex" % PREFIX, "splt.rarerindex")
         zf.write("%s.commonpairs" % PREFIX, "splt.commonpairs")
         zf.write("%s.kernel" % PREFIX, "splt.kernel")

         # these should be here..
         zf.write("%s.classifiers" % PREFIX, "splt.classifiers")
         zf.write("%s.fmap" % PREFIX, "splt.fmap")

         print "clearing leftovers"
         # remove..
         os.remove("%s.thrs" % PREFIX)
         os.remove("%s.vecs" % PREFIX)
         os.remove("%s.ais"  % PREFIX)
         os.remove("%s.common" % PREFIX)
         os.remove("%s.rarerindex" % PREFIX)
         os.remove("%s.commonpairs" % PREFIX)
         os.remove("%s.kernel" % PREFIX)
         os.remove("%s.classifiers" % PREFIX)
         os.remove("%s.fmap" % PREFIX)

# }}}

      

# TODO: verify poly-kernel deg2
def convert_and_write_multiclass_models(classifierDataInterface, out_prefix, rare_S, zip=False):
   model = classifierDataInterface
   if model.is_ovr():
      TYPE = "ovr" + ":" + "__OTHER__"
   elif model.is_pairwise():
      TYPE = "pairwise"
   else: 
      raise "Non-supported multiclass type for model in %s" % filename
   print "multiclass type:",TYPE

   META_OUT = file("%s.classifiers" % out_prefix,"w")
   META_OUT.write("type:%s\n" % TYPE)
   # features_map
   features_map = model.features_map()
   if features_map:
      fids = list(features_map.keys())
   else: fids = model.fids()

   kernel = model.kernel_type()
   degree = model.degree()
   gamma  = model.gamma()
   coef0  = model.coef0()

   write_features_map("%s.fmap" % out_prefix, features_map)

   labels = model.labels
   META_OUT.write("%s\n" % " ".join(labels))
   for classifier_id in xrange(model.nr_classifiers):
      truelbl,falselbl = model._cids_to_labels[classifier_id]
      #outname = "%s%s" % (out_prefix, i)
      META_OUT.write("%s %s %s\n" % (classifier_id, truelbl, falselbl))
   META_OUT.close()

   vecs = model.vecs()
   threshold = model._cid_to_thr
   fids = model.fids()
   outmodel = FastSVMDecoderData(vecs, threshold, fids, rare_S, degree,gamma,coef0 )
   outmodel.write_to_files(out_prefix,zip) # vectors, ais, rare-index, commons, pair-weights





   

      
